@@ -9,12 +9,16 @@ import json
 import logging
 import re
 import sys
+import traceback
 import uuid
 import warnings
 import zipfile
+from itertools import chain
+from multiprocessing import Pool
 from pathlib import Path
-from typing import Dict, Iterable, List, Mapping, Optional, Tuple
+from typing import Dict, Iterable, List, Mapping, Optional, Set, Tuple, Union
 
+import attr
 import click
 from defusedxml import minidom
 
@@ -181,7 +185,6 @@ def prepare_and_write(
     producer: str,
     embed_location: bool = None,
 ) -> Tuple[uuid.UUID, Path]:
-
     if embed_location is None:
         # Default to embedding the location if they're not in the same folder.
         embed_location = dataset_location.parent != output_yaml.parent
@@ -214,7 +217,9 @@ def prepare_and_write(
         p.instrument = "MSI"
         p.constellation = "sentinel-2"
 
-        p.dataset_version = f"1.0.{p.processed:%Y%m%d}"
+        # TODO: How to read collection number from metadata? (once ESA etc add one)
+        collection_number = 0
+        p.dataset_version = f"{collection_number}.0.{p.processed:%Y%m%d}"
 
         p.properties["odc:file_format"] = "JPEG2000"
         p.product_family = "level1"
@@ -377,6 +382,17 @@ class YearMonth(click.ParamType):
         return int(year), int(month)
 
 
+@attr.s(auto_attribs=True, order=True, hash=True)
+class Job:
+    """A dataset to process"""
+
+    dataset_path: Path
+    output_yaml_path: Path
+    # "sinergise.com" / "esa.int"
+    producer: str
+    embed_location: bool
+
+
 @click.command(help=__doc__)
 @click.option("-v", "--verbose", is_flag=True)
 @click.argument(
@@ -384,6 +400,23 @@ class YearMonth(click.ParamType):
     type=PathPath(exists=True, readable=True, writable=False, resolve_path=True),
     nargs=-1,
 )
+@click.option(
+    "-f",
+    "--datasets-path",
+    help="A file to read input dataset paths from, one per line",
+    required=False,
+    type=PathPath(
+        exists=True, readable=True, dir_okay=False, file_okay=True, resolve_path=True
+    ),
+)
+@click.option(
+    "-j",
+    "--jobs",
+    "workers",
+    help="Number of workers to run in parallel",
+    type=int,
+    default=1,
+)
 @click.option(
     "--overwrite-existing/--skip-existing",
     is_flag=True,
@@ -451,9 +484,11 @@ def main(
     output_base: Optional[Path],
     input_relative_to: Optional[Path],
     datasets: List[Path],
+    datasets_path: Optional[Path],
     provider: Optional[str],
     overwrite_existing: bool,
     verbose: bool,
+    workers: int,
     embed_location: Optional[bool],
     limit_regions_file: Optional[Path],
     before_month: Optional[Tuple[int, int]],
@@ -471,8 +506,14 @@ def main(
     if limit_regions_file:
         limit_regions = set(limit_regions_file.read_text().splitlines())
 
-    for i, input_path in enumerate(datasets):
+    extra_datasets = datasets_path.read_text().splitlines() if datasets_path else []
 
+    input_count = 0
+    jobs: Set[Job] = set()
+    for i, input_path in enumerate(
+        chain(datasets, (Path(p.strip()) for p in extra_datasets))
+    ):
+        input_count += 1
         info = FolderInfo.for_path(input_path)
 
         # Skip regions that are not in the limit?
@@ -568,12 +609,65 @@ def main(
 
                 _LOG.info("Output exists: overwriting %s", output_yaml)
 
-            uuid_, path = prepare_and_write(
-                ds_path, output_yaml, producer, embed_location=embed_location
-            )
-            _LOG.info("Wrote dataset %s to %s", uuid_, path)
+            jobs.add(Job(ds_path, output_yaml, producer, embed_location=embed_location))
 
-    sys.exit(0)
+    jobs = sorted(jobs)
+    _LOG.info(
+        f"{len(jobs)} dataset(s) to process out of {input_count} inputs. Using {workers} workers"
+    )
+
+    # If only one process, call it directly.
+    # (Multiprocessing makes debugging harder, so we prefer to make it optional)
+
+    errors = 0
+    if workers == 1:
+        for job in jobs:
+            try:
+                uuid_, path = prepare_and_write(
+                    job.dataset_path,
+                    job.output_yaml_path,
+                    job.producer,
+                    embed_location=job.embed_location,
+                )
+                _LOG.info("Wrote dataset %s to %s", uuid_, path)
+            except Exception:
+                _LOG.exception("Failed to write dataset: %s", job)
+                errors += 1
+    else:
+        with Pool(processes=workers) as pool:
+            for res in pool.imap_unordered(_write_dataset_safe, jobs):
+                if isinstance(res, str):
+                    _LOG.error(res)
+                    errors += 1
+                else:
+                    uuid_, path = res
+                    _LOG.info("Wrote dataset %s to %s", uuid_, path)
+            pool.close()
+            pool.join()
+
+    _LOG.debug(
+        f"Completed {len(jobs)-errors} dataset(s) successfully, with {errors} failure(s)"
+    )
+    sys.exit(errors)
+
+
+def _write_dataset_safe(job: Job) -> Union[Tuple[uuid.UUID, Path], str]:
+    """
+    A wrapper around `prepare_and_write` that catches exceptions and makes them
+    serialisable as error strings.
+
+    (for use in multiprocessing pools etc)
+    """
+    try:
+        uuid_, path = prepare_and_write(
+            job.dataset_path,
+            job.output_yaml_path,
+            job.producer,
+            embed_location=job.embed_location,
+        )
+        return uuid_, path
+    except Exception:
+        return f"Failed to write dataset: {job}\n" + traceback.format_exc()
 
 
 if __name__ == "__main__":
