@@ -4,12 +4,10 @@ from contextlib import contextmanager
 
 import pytest
 
-from eodatasets3 import namer, Eo3Dict
+from eodatasets3 import Eo3Dict, namer
 from eodatasets3.model import DatasetDoc
 from eodatasets3.names import NamingConventions
-from eodatasets3.properties import (
-    PropertyOverrideWarning,
-)
+from eodatasets3.properties import PropertyOverrideWarning
 
 
 def test_multi_platform_fields():
